@@ -10,14 +10,17 @@ List Fastcpd::negative_log_likelihood(
     bool cv,
     Nullable<colvec> start
 ) {
+  List result;
   if (theta.isNull()) {
-    return negative_log_likelihood_wo_theta(data, lambda, cv, start);
+    result = negative_log_likelihood_wo_theta(data, lambda, cv, start);
   } else {
-    return List::create(
+    result = List::create(
       Named("value") =
         negative_log_likelihood_wo_cv(data, as<colvec>(theta), lambda)
     );
   }
+  result["value"] = adjust_cost_value(result["value"], data.n_rows);
+  return result;
 }
 
 List Fastcpd::negative_log_likelihood_wo_theta(
@@ -73,9 +76,11 @@ List Fastcpd::negative_log_likelihood_wo_theta(
       predict_glmnet(out, data.cols(1, data.n_cols - 1), Named("s") = lambda)
     );
     vec residuals = data.col(0) - fitted_values;
-    return List::create(Named("par") = par,
-                  Named("value") = value / 2,
-                  Named("residuals") = residuals);
+    return List::create(
+      Named("par") = par,
+      Named("value") = value / 2,
+      Named("residuals") = residuals
+    );
   } else if (
     family == "binomial" || family == "poisson" || family == "gaussian"
   ) {
@@ -94,15 +99,17 @@ List Fastcpd::negative_log_likelihood_wo_theta(
     vec par = as<vec>(out["coefficients"]);
     vec residuals = as<vec>(out["residuals"]);
     double value = out["deviance"];
-    return List::create(Named("par") = par,
-                  Named("value") = value / 2,
-                  Named("residuals") = residuals);
+    return List::create(
+      Named("par") = par,
+      Named("value") = value / 2,
+      Named("residuals") = residuals
+    );
   } else if (family == "arma") {
     Environment stats = Environment::namespace_env("stats");
     Function arima = stats["arima"];
     List out = arima(
       Named("x") = data.col(0),
-      Named("order") = Rcpp::NumericVector::create(order(0), 0, order(1)),
+      Named("order") = NumericVector::create(order(0), 0, order(1)),
       Named("include.mean") = false
     );
     colvec par = zeros(sum(order) + 1);
@@ -114,42 +121,38 @@ List Fastcpd::negative_log_likelihood_wo_theta(
                   Named("residuals") = as<vec>(out["residuals"]));
   } else if (family == "mean") {
     rowvec par = mean(data, 0);
-    mat residuals = data.each_row() - par;
     return List::create(
       Named("par") = par,
       Named("value") = data.n_rows / 2.0 * (
-        log(2.0 * M_PI) * data.n_cols + log_det_sympd(mean_data_cov) +
-          arma::trace(
-            solve(mean_data_cov, residuals.t() * residuals)
-          ) / data.n_rows
-      ) + data.n_cols * log(data.n_rows),
-      Named("residuals") = residuals
+        std::log(2.0 * M_PI) * data.n_cols + log_det_sympd(variance_estimate) +
+          trace(solve(
+            variance_estimate,
+            (data.each_row() - par).t() * (data.each_row() - par)
+          )) / data.n_rows
+      ),
+      Named("residuals") = data.each_row() - par
     );
   } else if (family == "variance") {
     mat residuals = data.each_row() - variance_data_mean;
-    mat par = residuals.t() * residuals / (data.n_rows - 1);
-    double value =
-      data.n_cols * (log(2.0 * M_PI) + (data.n_rows - 1) / data.n_rows);
+    mat par = residuals.t() * residuals / data.n_rows;
+    double value = data.n_rows * data.n_cols * (std::log(2.0 * M_PI) + 1) / 2.0;
     if (data.n_rows >= data.n_cols) {
-      value += log_det_sympd(par);
+      value += data.n_rows * log_det_sympd(par) / 2.0;
     }
-    value *= data.n_rows / 2.0;
     return List::create(
       Named("par") = par,
       Named("value") = value,
       Named("residuals") = residuals
     );
   } else if (family == "meanvariance" || family == "mv") {
-    mat covariance = arma::cov(data);
+    mat covariance = cov(data);
 
-    double value =
-      data.n_cols * (log(2.0 * M_PI) + (data.n_rows - 1) / data.n_rows);
+    double value = data.n_rows * data.n_cols * (std::log(2.0 * M_PI) + 1) / 2.0;
     if (data.n_rows >= data.n_cols) {
-      value += log_det_sympd(
+      value += data.n_rows * log_det_sympd(
         covariance + epsilon * eye<mat>(data.n_cols, data.n_cols)
-      );
+      ) / 2.0;
     }
-    value *= data.n_rows / 2.0;
 
     colvec par = zeros(data.n_cols * data.n_cols + data.n_cols);
     par.rows(0, data.n_cols - 1) = mean(data, 0).t();
@@ -162,9 +165,33 @@ List Fastcpd::negative_log_likelihood_wo_theta(
       Named("value") = value,
       Named("residuals") = residuals
     );
+  } else if (family == "mgaussian") {
+    mat x = data.cols(p_response, data.n_cols - 1);
+    mat y = data.cols(0, p_response - 1);
+    mat x_t_x;
+
+    if (data.n_rows <= data.n_cols - p_response + 1) {
+      x_t_x = eye<mat>(data.n_cols - p_response, data.n_cols - p_response);
+    } else {
+      x_t_x = x.t() * x;
+    }
+
+    mat par = solve(x_t_x, x.t()) * y;
+    DEBUG_RCOUT(par);
+    mat residuals = y - x * par;
+    double value =
+      p_response * std::log(2.0 * M_PI) + log_det_sympd(variance_estimate);
+    value *= data.n_rows;
+    value += trace(solve(variance_estimate, residuals.t() * residuals));
+    DEBUG_RCOUT(value);
+    return List::create(
+      Named("par") = par,
+      Named("value") = value / 2,
+      Named("residuals") = residuals
+    );
   } else {
     // # nocov start
-    stop("This branch should not be reached at functions.cc: 103.");
+    stop("This branch should not be reached at fastcpd_class_cost.cc: 193.");
     // # nocov end
   }
 }
@@ -177,9 +204,9 @@ double Fastcpd::negative_log_likelihood_wo_cv(
   vec y = data.col(0);
   if (family == "lasso" || family == "gaussian") {
     // Calculate negative log likelihood in gaussian family
-    double penalty = lambda * accu(arma::abs(theta));
+    double penalty = lambda * accu(abs(theta));
     mat x = data.cols(1, data.n_cols - 1);
-    return accu(arma::pow(y - x * theta, 2)) / 2 + penalty;
+    return accu(square(y - x * theta)) / 2 + penalty;
   } else if (family == "binomial") {
     // Calculate negative log likelihood in binomial family
     mat x = data.cols(1, data.n_cols - 1);
@@ -199,26 +226,26 @@ double Fastcpd::negative_log_likelihood_wo_cv(
 
     return accu(-y % u + exp(u) + y_factorial);
   } else if (family == "arma") {
-    colvec reversed_theta = arma::reverse(theta);
+    colvec reversed_theta = reverse(theta);
     if (data.n_rows < max(order) + 1) {
       return 0;
     }
     colvec variance_term = zeros(data.n_rows);
     for (unsigned int i = max(order); i < data.n_rows; i++) {
-      variance_term(i) = data(i, 0) - arma::dot(
+      variance_term(i) = data(i, 0) - dot(
           reversed_theta.rows(order(1) + 1, sum(order)),
           data.rows(i - order(0), i - 1)
-        ) - arma::dot(
+        ) - dot(
           reversed_theta.rows(1, order(1)),
           variance_term.rows(i - order(1), i - 1)
         );
     }
-    return (log(2.0 * M_PI) +
-      log(theta(sum(order)))) * (data.n_rows - 2) / 2.0 +
-      arma::dot(variance_term, variance_term) / 2.0 / theta(sum(order));
+    return (std::log(2.0 * M_PI) +
+      std::log(theta(sum(order)))) * (data.n_rows - 2) / 2.0 +
+      dot(variance_term, variance_term) / 2.0 / theta(sum(order));
   } else {
     // # nocov start
-    stop("This branch should not be reached at functions.cc: 153.");
+    stop("This branch should not be reached at functions.cc: 248.");
     // # nocov end
   }
 }
@@ -235,22 +262,22 @@ colvec Fastcpd::cost_update_gradient(mat data, colvec theta) {
   } else if (family == "lasso" || family == "gaussian") {
     gradient = - (y - as_scalar(x * theta)) * x.t();
   } else if (family == "arma") {
-    mat reversed_data = arma::reverse(data, 0);
-    colvec reversed_theta = arma::reverse(theta);
+    mat reversed_data = reverse(data, 0);
+    colvec reversed_theta = reverse(theta);
     if (data.n_rows < max(order) + 1) {
-      return arma::ones(theta.n_elem);
+      return ones(theta.n_elem);
     }
     colvec variance_term = zeros(data.n_rows);
     for (unsigned int i = max(order); i < data.n_rows; i++) {
-      variance_term(i) = data(i, 0) - arma::dot(
+      variance_term(i) = data(i, 0) - dot(
           reversed_theta.rows(order(1) + 1, sum(order)),
           data.rows(i - order(0), i - 1)
-        ) - arma::dot(
+        ) - dot(
           reversed_theta.rows(1, order(1)),
           variance_term.rows(i - order(1), i - 1)
         );
     }
-    colvec reversed_variance_term = arma::reverse(variance_term);
+    colvec reversed_variance_term = reverse(variance_term);
     mat phi_coefficient = zeros(data.n_rows, order(0)),
         psi_coefficient = zeros(data.n_rows, order(1));
     for (unsigned int i = max(order); i < data.n_rows; i++) {
@@ -291,27 +318,28 @@ mat Fastcpd::cost_update_hessian(mat data, colvec theta) {
     hessian = (x.t() * x) * as_scalar((1 - prob) * prob);
   } else if (family.compare("poisson") == 0) {
     double prob = exp(as_scalar(x * theta));
-    hessian = (x.t() * x) * std::min(as_scalar(prob), min_prob);
+    // Prevent numerical issues if `prob` is too large.
+    hessian = (x.t() * x) * std::min(as_scalar(prob), 1e10);
   } else if (family == "lasso" || family == "gaussian") {
     hessian = x.t() * x;
   } else if (family == "arma") {
     // TODO(doccstat): Maybe we can store all these computations
-    mat reversed_data = arma::reverse(data, 0);
-    colvec reversed_theta = arma::reverse(theta);
+    mat reversed_data = reverse(data, 0);
+    colvec reversed_theta = reverse(theta);
     if (data.n_rows < max(order) + 1) {
-      return arma::eye(theta.n_elem, theta.n_elem);
+      return eye(theta.n_elem, theta.n_elem);
     }
     colvec variance_term = zeros(data.n_rows);
     for (unsigned int i = max(order); i < data.n_rows; i++) {
-      variance_term(i) = data(i, 0) - arma::dot(
+      variance_term(i) = data(i, 0) - dot(
           reversed_theta.rows(order(1) + 1, sum(order)),
           data.rows(i - order(0), i - 1)
-        ) - arma::dot(
+        ) - dot(
           reversed_theta.rows(1, order(1)),
           variance_term.rows(i - order(1), i - 1)
         );
     }
-    colvec reversed_variance_term = arma::reverse(variance_term);
+    colvec reversed_variance_term = reverse(variance_term);
     mat phi_coefficient = zeros(data.n_rows, order(0)),
         psi_coefficient = zeros(data.n_rows, order(1));
     for (unsigned int i = max(order); i < data.n_rows; i++) {
@@ -326,8 +354,8 @@ mat Fastcpd::cost_update_hessian(mat data, colvec theta) {
         ).t() - reversed_theta.rows(1, order(1)).t() *
         psi_coefficient.rows(i - order(1), i - 1);
     }
-    mat reversed_coef_phi = arma::reverse(phi_coefficient, 0),
-        reversed_coef_psi = arma::reverse(psi_coefficient, 0);
+    mat reversed_coef_phi = reverse(phi_coefficient, 0),
+        reversed_coef_psi = reverse(psi_coefficient, 0);
     cube phi_psi_coefficient = zeros(order(1), order(0), data.n_rows),
          psi_psi_coefficient = zeros(order(1), order(1), data.n_rows);
     for (unsigned int i = order(1); i < data.n_rows; i++) {
