@@ -11,7 +11,12 @@ List CostFunction::operator()(  // # nocov
     bool cv,  // UNUSED
     Nullable<colvec> start  // UNUSED
 ) {
-  return theta.isNull()? cost(data) : cost(data, theta);  // # nocov
+  DEBUG_RCOUT(data.n_rows);
+  SEXP value =
+    theta.isNull() ? cost(data) : cost(data, as<colvec>(theta));  // # nocov
+  return List::create(
+      Named("value") = as<double>(value)  // # nocov
+  );
 }
 
 CostGradient::CostGradient(Function cost_gradient) :
