@@ -3,29 +3,53 @@
 
 #include "RcppArmadillo.h"
 
+#ifdef DEBUG
+#  define DEBUG_RCOUT(x) Rcout << #x << ": " << x << std::endl
+#else
+#  define DEBUG_RCOUT(x) do {} while (0)
+#endif
+
+using ::arma::abs;
 using ::arma::accu;
+using ::arma::approx_equal;
 using ::arma::as_scalar;
 using ::arma::clamp;
+using ::arma::cov;
+using ::arma::diff;
+using ::arma::dot;
+using ::arma::find;
 using ::arma::eye;
+using ::arma::floor;
 using ::arma::index_min;
 using ::arma::join_cols;
+using ::arma::join_rows;
+using ::arma::join_slices;
 using ::arma::linspace;
 using ::arma::log_det_sympd;
 using ::arma::max;
+using ::arma::mean;
 using ::arma::min;
 using ::arma::norm;
 using ::arma::ones;
+using ::arma::reverse;
 using ::arma::sign;
 using ::arma::solve;
+using ::arma::sort;
+using ::arma::square;
+using ::arma::trace;
+using ::arma::unique;
 using ::arma::zeros;
 using ::Rcpp::as;
+using ::Rcpp::checkUserInterrupt;
 using ::Rcpp::Named;
 using ::Rcpp::stop;
+using ::Rcpp::wrap;
 
 using ::arma::colvec;
 using ::arma::cube;
 using ::arma::mat;
 using ::arma::rowvec;
+using ::arma::span;
 using ::arma::ucolvec;
 using ::arma::uvec;
 using ::arma::vec;
@@ -39,4 +63,6 @@ using ::Rcpp::S4;
 using ::std::function;
 using ::std::string;
 
+using ::Rcpp::Rcout;
+
 #endif  // FASTCPD_TYPES_H_
